import { MediaPlayer } from 'dashjs';
import queryString from 'query-string';

import style from './player.styl';

// Nonce used by Webpack for CSP
// eslint-disable-next-line
const __webpack_nonce__ = 'uGo4I9ydb2hP393boc/24Vu7diUk/Mf84w9khcZkynk=';

function getSecondsOfDay() {
  const date = new Date();
  const localTime = date.getTime() - (1000 * 60 * date.getTimezoneOffset());
  return (localTime % (1000 * 60 * 60 * 24)) / 1000;
}

window.addEventListener('load', () => {
  const playContainer = document.createElement('div');
  playContainer.classList.add(style.play);
  const playButton = document.createElement('button');
  playButton.classList.add('icon');
  playButton.classList.add('icon-play'); // fontello icon
  playContainer.appendChild(playButton);

  const statusContainer = document.createElement('div');
  statusContainer.classList.add(style.status);
  const statusContent = document.createElement('span');
  statusContent.classList.add(style.statusInner);
  statusContainer.appendChild(statusContent);

  function showStatus() {
    if (!(style['status--visible'] in statusContainer.classList)) {
      statusContainer.classList.add(style['status--visible']);
    }
  }

  function setStatus(text) {
    statusContent.innerText = text;
  }

  function hideStatus() {
    statusContainer.classList.remove(style['status--visible']);
  }

  function showPlayButton() {
    if (!(style['play--visible'] in playContainer.classList)) {
      playContainer.classList.add(style['play--visible']);
    }
  }

  function hidePlayButton() {
    playContainer.classList.remove(style['play--visible']);
  }

  showStatus('Constructing your clock...');

  const playerContainer = document.createElement('video');
  playerContainer.classList.add(style.video);
  playerContainer.loop = true;
  playerContainer.controls = false;

  const parsedHash = queryString.parse(window.location.query || window.location.hash);
  let secondsOfDay;
  if (parsedHash.s !== undefined) {
    // use query-provided start time
    secondsOfDay = parseInt(parsedHash.s, 10);
  } else {
    // calculate start time
    secondsOfDay = getSecondsOfDay();
  }

  // initialize video
  const player = MediaPlayer().create();
  const url = `video/clock.mpd#s=${secondsOfDay}`;
  const autoplay = true;

  playButton.addEventListener('click', () => {
    hidePlayButton();

    // resync video position with current time if necessary
    if (parsedHash.s === undefined) {
      player.seek(getSecondsOfDay());
    }

    player.play();
  });

  player.on('error', (e) => {
    showStatus();
    setStatus(`Error: ${e}`);
  });

  player.on('bufferEmpty', () => {
    setStatus('We ran out of wood!');
  });

  player.on('bufferStalled', () => {
    setStatus('Waiting for the workers to arrive…');
  });

  player.on('bufferLoaded', () => {
    setStatus('Almost there…');
  });

  player.on('playbackWaiting', () => {
    showStatus();
    setStatus('We are waiting for more wood…');
  });

  player.on('canPlay', () => {
    hideStatus();
    showPlayButton();
  });

  player.on('playbackPlaying', () => {
    hideStatus();
    hidePlayButton();

    // resync video position with current time if necessary
    if (parsedHash.s === undefined && Math.abs(player.time() - getSecondsOfDay()) > 1) {
      player.seek(getSecondsOfDay());
    }
  });

  document.body.appendChild(playerContainer);
  document.body.appendChild(playContainer);
  document.body.appendChild(statusContainer);

  player.initialize(playerContainer, url, autoplay);
});
